;;; swirls-mode.el - major mode for playing sega swirls
;;;
;;; Dennis Marti
;;; April 6, 2002 (Happy Birthday David)
;;;
;;; $Id: swirls-mode.el,v 1.6 2002/04/12 03:31:00 dennis Exp dennis $

(require 'cl)

(defvar swirls-mode-hook nil
  "List of functions to call when starting swirls.")

(defvar swirls-mode-map nil
  "Key map for swirls major mode.")

(if swirls-mode-map
    t
  (setq swirls-mode-map (make-keymap))
  ;(dotimes (c 255) (define-key swirls-mode-map (format "%c" c) 'print))
    
  (define-key swirls-mode-map "\C-J" 'swirls-delete-pieces)
  (define-key swirls-mode-map "\C-M" 'swirls-delete-pieces)
  (define-key swirls-mode-map "\C-d" 'swirls-delete-pieces)
  (define-key swirls-mode-map " " 'swirls-delete-pieces)

  (define-key swirls-mode-map "q" 'kill-buffer)
  (define-key swirls-mode-map "n" 'swirls)
  (define-key swirls-mode-map "r" 'swirls-rotate-pieces)
  (define-key swirls-mode-map "s" 'swirls-display-selected-pieces)
  (define-key swirls-mode-map "w" 'swirls-set-board)

  (define-key swirls-mode-map "R" 'swirls-reset-scores)

  ;; XXX how do you do arrow keys?
  (define-key swirls-mode-map "OB" 'swirls-down-row)
  (define-key swirls-mode-map "\C-n" 'swirls-down-row)
  (define-key swirls-mode-map "OA" 'swirls-up-row)
  (define-key swirls-mode-map "\C-p" 'swirls-up-row)
  (define-key swirls-mode-map "OD" 'swirls-left-col)
  (define-key swirls-mode-map "\C-b" 'swirls-left-col)
  (define-key swirls-mode-map "OC" 'swirls-right-col)
  (define-key swirls-mode-map "\C-f" 'swirls-right-col))

(defun swirls-mode ()
  (interactive)
  (kill-all-local-variables)
  (setq major-mode 'swirls-mode)
  (setq mode-name "Swirls")
  (use-local-map swirls-mode-map)
  (run-hooks 'swirls-mode-hook))

(provide 'swirls)

;; XXX how do you make these buffer-local? should you?
(defvar *swirls-board*)
(defvar *swirls-position*)
(defvar *swirls-start-position* 0)
(defvar *swirls-board-size* 10)
(defvar *swirls-empty-piece* 0)

(defvar swirls-score 0)
(defvar swirls-longest 0)
(defvar swirls-moves 0)
(defvar swirls-singles 0)
(defvar swirls-best-score 0)
(defvar swirls-best-longest 0)
(defvar swirls-best-moves 0)
(defvar swirls-best-singles 0)

(defvar *swirls-pieces* '("   " "() " "[] " "{} " "<> "))
;(setq *swirls-pieces* '("   " "() " "[] " "{} " "<> "))
;(setq *swirls-pieces* '("  " "@ " "+ " "A " "[ "))
;(setq *swirls-pieces* '("    " "[@] " "[+] " "[/] " "[ ] "))
;(setq *swirls-pieces* '("    " "(@) " "[X] " "[O] " "( ) "))
;(setq *swirls-pieces* '("    " "(=) " "{!} " "[@] " "<X> ")
;(setq *swirls-pieces* '("  " "X " "O " "@ " "= "))
;(setq *swirls-pieces* '("    " "### " "||| " "[X] " "(+) "))
;(setq *swirls-pieces* '("    " "### " "||| " "[X] " "(:) "))
(setq *swirls-pieces* '("    " "### " "||| " "::: " "(+) "))

(defvar *swirls-toggle-pieces* '("    " "XXX " "/// "))
;(setq *swirls-toggle-pieces* '("    " "XXX " "/// "))

(defun swirls-rotate-pieces ()
  "Rotate the available pieces."
  (interactive)
  (setq *swirls-pieces*
	(cons (car *swirls-pieces*)
	      (append (last (cdr *swirls-pieces*))
		      (butlast (cdr *swirls-pieces*)))))
  (swirls-display-board *swirls-board*))

(defun swirls-col (n) (mod n *swirls-board-size*))

(defun swirls-row (n) (/ n *swirls-board-size*))

(defun swirls-board-size ()
  (* *swirls-board-size* *swirls-board-size*))

(defun swirls-set-position (&optional n)
  (cond
   ((null n) (swirls-set-position *swirls-position*))
   ((< n 0) (swirls-set-position (+ n (swirls-board-size))))
   ((>= n (swirls-board-size))
    (swirls-set-position (- n (swirls-board-size))))
   (t
    (beginning-of-buffer)
    (move-to-window-line (1+ (swirls-row n)))
    (move-to-column (+ 8 (* (length (car *swirls-pieces*)) (swirls-col n))))
    (setq *swirls-position* n))))

(defun swirls-up-row ()
  "Move up a row on the board."
  (interactive)
  (swirls-set-position (- *swirls-position* *swirls-board-size*)))

(defun swirls-down-row ()
  "Move down a row on the board."
  (interactive)
  (swirls-set-position (+ *swirls-position* *swirls-board-size*)))

(defun swirls-left-col ()
  "Move on column to the left."
  (interactive)
  (swirls-set-position (1- *swirls-position*)))

(defun swirls-right-col ()
  "Move one column to the right."
  (interactive)
  (swirls-set-position (1+ *swirls-position*)))

(defun swirls-score-string (a b)
  (format "%6d %6d" a b))

(defun swirls-show-piece-count (piece)
  (format "\t    %s  %5d" (or (nth piece *swirls-pieces*) (car *swirls-pieces*))
	  (let ((total 0))
	    (dotimes (row *swirls-board-size* total)
	      (dotimes (col *swirls-board-size*)
		(if (= piece (swirls-get-piece *swirls-board* row col))
		    (incf total)))))))

(defun swirls-display-score ()
  (insert "\n"
	  "\n\t\tcurrent   best     piece  count"
	  "\n\tscore:\t "
	  (swirls-score-string swirls-score swirls-best-score)
	  (swirls-show-piece-count 1)
	  "\n\tmoves:\t "
	  (swirls-score-string swirls-moves swirls-best-moves)
	  (swirls-show-piece-count 2)
	  "\n\tlongest: "
	  (swirls-score-string swirls-longest swirls-best-longest)
	  (swirls-show-piece-count 3)
	  "\n\tsingles: "
	  (swirls-score-string swirls-singles swirls-best-singles)
	  (swirls-show-piece-count 4)
	  ))

(defun swirls-get-piece (a i j)
  (aref (aref a i) j))

(defun swirls-set-piece (a i j v)
  (let ((inner (aref a i)))
    (aset inner j v)))

;; all these gyrations compacting columns makes me wonder if the board
;; should be set up the other way (col x row).

(defun swirls-compact-column (b c)
  (let ((acc ()))
    (dotimes (row *swirls-board-size*)
      (let ((p (swirls-get-piece b row c)))
	(unless (zerop p) (push p acc))))
    (do ((row (1- *swirls-board-size*) (1- row)))
	((< row 0))
      (swirls-set-piece b row c (if acc (pop acc) 0)))))

(defun swirls-empty-column-p (b c)
  (let ((ans t))
    (dotimes (i *swirls-board-size* ans)
      (unless (zerop (swirls-get-piece b i c))
	(setq ans nil)))))

(defun swirls-swap-columns (b c1 c2)
  "we only swap when c1 is empty, so we don't need to remember c1 values"
  (dotimes (row *swirls-board-size*)
    (swirls-set-piece b row c1 (swirls-get-piece b row c2))
    (swirls-set-piece b row c2 0)))

(defun swirls-next-non-empty-column (board col)
  (cond
   ((= col *swirls-board-size*) nil)
   ((swirls-empty-column-p board col) 
    (swirls-next-non-empty-column board (1+ col)))
   (t col)))

(defun swirls-compact-board (b)
  "first fill in any zeros with pieces above. then move remove empty columns."
  (dotimes (col *swirls-board-size*)
    (swirls-compact-column b col))
  (dotimes (col (1- *swirls-board-size*))
    (if (swirls-empty-column-p b col)
	(let ((n (swirls-next-non-empty-column b (1+ col))))
	  (if n (swirls-swap-columns b col n))))))

(defun swirls-display-board (b &optional do-not-compact)
  (unless do-not-compact
    (swirls-compact-board b))
  (erase-buffer)
  (dotimes (i *swirls-board-size*)
    (insert "\n\t")
    (dotimes (j *swirls-board-size*)
      (insert (nth (swirls-get-piece b i j) *swirls-pieces*))))
  (swirls-display-score)
  (swirls-set-position *swirls-position*))

(defvar swirls-display-selected-toggle t)

(defun swirls-display-selected-pieces ()
  "Display only the pieces that would be removed if you deleted now."
  ;; XXX update this so it shows like pieces that aren't in the current
  ;; XXX  selection.
  (interactive)
  (setq swirls-display-selected-toggle (not swirls-display-selected-toggle))
  (if swirls-display-selected-toggle
      (swirls-display-board *swirls-board*)
    (let ((p (swirls-get-pieces-like-point 'do-not-score))
	  (*swirls-pieces* *swirls-toggle-pieces*))
      (let ((b (swirls-init-board
		(lambda (row col)
		  (cond
		   ((member (+ col (* row *swirls-board-size*)) p)
		    1)
		   ((= (swirls-get-piece-pos *swirls-position*)
		       (swirls-get-piece *swirls-board* row col))
		    2)
		   (t 0))))))
	(swirls-display-board b 'do-not-compact)))))


(defun swirls-random-filler (row col)
  "initial function to fill board with pieces. chooses a random piece."
  (1+ (random (1- (length *swirls-pieces*)))))

(defun swirls-init-board (filler-function)
  (let ((acc ()))
    (dotimes (i *swirls-board-size* (apply 'vector (nreverse acc)))
      (let ((acc2 ()))
	(push (dotimes (j *swirls-board-size* (apply 'vector (nreverse acc2)))
		(push (funcall filler-function i j) acc2))
	      acc)))))

(defun swirls-set-piece-pos (piece pos)
  (let ((v (aref *swirls-board* (swirls-row pos))))
    (aset v (swirls-col pos) piece)))

(defun swirls-get-piece-pos (pos)
  (cond
   ((< pos 0) nil)
   ((>= pos (swirls-board-size)) nil)
   (t
    (swirls-get-piece *swirls-board* (swirls-row pos) (swirls-col pos)))))

(defun swirls-connected-p (board a b checked)
  "return true if there's a path of like pieces connecting positions a and b."
  (let ((p (swirls-get-piece-pos a)))
    (cond
     ((= a b) t)
     ((memq a checked) nil)
     ((and (eql (swirls-col a) (swirls-col (- a *swirls-board-size*)))
	   (eql p (swirls-get-piece-pos (- a *swirls-board-size*)))
	   (swirls-connected-p board (- a *swirls-board-size*) b (cons a checked)))
      t)
     ((and (eql (swirls-col a) (swirls-col (+ a *swirls-board-size*)))
	   (eql p (swirls-get-piece-pos (+ a *swirls-board-size*)))
	   (swirls-connected-p board (+ a *swirls-board-size*) b (cons a checked)))
      t)
     ((and (eql (swirls-row a) (swirls-row (1- a)))
	   (eql p (swirls-get-piece-pos (1- a)))
	   (swirls-connected-p board (1- a) b (cons a checked)))
      t)
     ((and (eql (swirls-row a) (swirls-row (1+ a)))
	   (eql p (swirls-get-piece-pos (1+ a)))
	   (swirls-connected-p board (1+ a) b (cons a checked)))
      t)
     (t nil))))

(defun swirls-score (n)
  (cond
   ((= 1 n) -50)
   ((< n 25) (* n n))
   (t (* 25 n))))

;(swirls-score 25)

(defun swirls-get-pieces-like-point (&optional do-not-score)
  (let ((p (swirls-get-piece-pos *swirls-position*))
	(acc ()))
    ;; get the positions of all pieces like point (including point)
    (if (zerop p)
	nil
      (dotimes (i (swirls-board-size))
	(if (= p (swirls-get-piece-pos i))
	    (if (swirls-connected-p *swirls-board* i *swirls-position* (list))
		(push i acc))))
      ;; do the scoring
      (unless do-not-score
	(let  ((len (length acc)))
	  (incf swirls-score (swirls-score len))
	  (incf swirls-moves)
	  (setq swirls-longest
		(if (> len swirls-longest) len swirls-longest))
	  (if (= 1 len) (incf swirls-singles))))
      acc)))

(defun swirls-delete-pieces ()
  "Delete the piece at the point and the adjacent ones like it."
  (interactive)
  (dolist (p (swirls-get-pieces-like-point))
    (swirls-set-piece-pos *swirls-empty-piece* p))
  (swirls-set-piece-pos *swirls-empty-piece* *swirls-position*)
  (swirls-display-board *swirls-board*))

(defun swirls-set-board (n)
  "Change the size of the board. Only square boards are supported."
  (interactive "nboard width: " )
  (setq *swirls-board-size* n))

(defun swirls-set-best ()
  (when (> swirls-score swirls-best-score)
    (setq swirls-best-score    swirls-score
	  swirls-best-longest  swirls-longest
	  swirls-best-moves    swirls-moves
	  swirls-best-singles  swirls-singles)))

(defun swirls-reset-scores ()
  "Reset the high scores."
  (interactive)
  (setq swirls-best-score    0
	swirls-best-longest  0
	swirls-best-moves    0
	swirls-best-singles  0)
  (swirls-display-board *swirls-board*))

(defun swirls ()
  "Start a new swirls game."
  (interactive)
  (let ((this-buff (switch-to-buffer "*swirls*")))
    (let ((default-major-mode 'swirls-mode))
      (print "setting-buffer-major-mode")
      (print this-buff)
      (set-buffer-major-mode this-buff)
      (swirls-set-best)
      (setq *swirls-board* (swirls-init-board 'swirls-random-filler)
	    *swirls-position* 0
	    swirls-score 0
	    swirls-moves 0
	    swirls-singles 0
	    swirls-longest 0)
	(swirls-display-board *swirls-board*)
	(swirls-set-position *swirls-start-position*))))

;;; end of swirls.el
