;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  File : 	  gui.emacs.el
;;
;;  Purpose: 	  call extensions that I would want to use only in the GUI
;;                version of GNU Emacs
;;        	
;;  Author:	  Steve Russell  September 18th 2001
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Add Directories For My Customization Collections To The Load Path
;; ~ means relative to $HOME dir

(add-to-list 'load-path "~/Emacs/Ext") 
(add-to-list 'load-path "~/Emacs/Ext/ecb-1.70")
(add-to-list 'load-path "~/Emacs/Ext/eieio-0.16")
(add-to-list 'load-path "~/Emacs/Ext/semantic-1.4")

;; Make the right mouse button display the edit menu
(global-set-key [mouse-3] menu-bar-edit-menu)



;; CUA mode 
;; 
;; makes emacs keybindings behave like the standard keybindings in
;; micrsoft windows, motif, macintosh & kde applications.
;;
;; ie : C-v paste, C-z undo etc
;;
;;
;; CUA mode is smart.  It knows that C-x & C-c are prefixes to emacs commands.
;; It will not use these keys as CUA keys unless a region is selected.  This
;; still leaves some holes you can get around by doing any of the 3 options:
;;
;; 1.  press the prefix key twice very quickly (within 0.2 seconds),
;; 2.  press the prefix key and the following key within 0.2 seconds), or
;; 3.  use the SHIFT key with the prefix key, i.e. C-X or C-C
;;
;; Instead of using the standard emacs keybindings for rectangle editing use
;; S-return ( shift enter ) to start a rectangle.
;;
;; Then use C-x, C-c, C-v to cut, copy or paste that rectangle

(require 'cua)
(CUA-mode t)


;; Semantic - dependancy for the Emacs Code Browser
;;(setq semantic-load-turn-everything-on t)
;;(require 'semantic-load)

;; Emacs Code Browser - a GUI directory/file/code tree
(require 'ecb)

;; Start The Emacs Code Browser On Start Up
;;(ecb-activate)

;; Line Numbers
(load-file "~/Emacs/Ext/setnu.el")

;; Add On Menu
(load-file "~/Emacs/Ext/emacsMenu.el")

;; If Emacs is running on Microsoft Windows load Windows preferences
(if (eq window-system 'w32)
    (load-file "~/Emacs/w32.emacs.el"))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                       END OF FILE:  loadGUIExt.el
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

